import React from 'react'
const {useReducer, useEffect} = React
import {useMemoOne, useCallbackOne} from 'use-memo-one'

function _ref(prev, action) {
  return {
    status: action.status,
    value: action.status === 'success' ? action.value : void 0,
    error: action.status === 'error' ? action.error : void 0,
  }
}

export function useAsync(asyncCallback, dependencies = []) {
  const [state, dispatch] = useReducer(_ref, {
    status: 'idle',
    value: void 0,
    error: void 0,
  }) // Creates a stable callback that manages our loading/success/error status updates
  // as the callback is invoked.

  const callback = useCallbackOne(
    Object.assign(
      async (...args) => {
        callback.cancelled = false
        dispatch({
          status: 'loading',
        })

        try {
          const value = await asyncCallback(...args)
          if (callback.cancelled) return
          dispatch({
            status: 'success',
            value,
          })
        } catch (error) {
          if (callback.cancelled) return
          dispatch({
            status: 'error',
            error,
          })
        }
      },
      {
        cancelled: false,
      }
    ), // eslint-disable-next-line react-hooks/exhaustive-deps
    dependencies
  ) // Cancel any pending async stuff on unmount

  function _ref2() {
    callback.cancelled = true
  }

  useEffect(() => _ref2, [callback])

  function _ref3() {
    callback.cancelled = true
    dispatch({
      status: 'cancelled',
    })
  }

  return [
    useMemoOne(
      () =>
        Object.assign(
          {
            cancel: _ref3,
          },
          state
        ),
      [callback, state]
    ),
    callback,
  ]
}
export function useAsyncEffect(asyncCallback, dependencies) {
  const [state, callback] = useAsync(
    asyncCallback,
    dependencies || [Math.random()]
  ) // Runs the callback each time the dependencies change and cancels any
  // pending callbacks that were running previously

  useEffect(
    () => {
      callback()
      return state.cancel
    }, // eslint-disable-next-line react-hooks/exhaustive-deps
    dependencies
  )
  return state
}
